/* z_rs.h (ZRAM: Reverse search)
   Ambros Marzetta
   95-06-26

 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_rs.h,v $
 * $Revision: 1.3 $
 * $Date: 1996/05/29 11:08:22 $
 * $Log: z_rs.h,v $
 * Revision 1.3  1996/05/29  11:08:22  marzetta
 * Changed some types
 *
 * Revision 1.1  1995/07/24  11:21:24  marzetta
 * Initial revision
 */

#ifndef _Z_RS_H_
#define _Z_RS_H_
#include "z_base.h"
#include "z_ps.h"
#include "z_lb.h"

typedef z_NodeP (*z_rs_FProc)(z_lb_GlobalP global, z_NodeP node);
/* Local search function.
   Generate and allocate a node's parent in the reverse search tree.
   It is never called for the root of the tree. */

typedef uint32 (*z_rs_DeltaProc)(z_lb_GlobalP global, z_NodeP node);

typedef z_NodeP (*z_rs_AdjOracleProc)(z_lb_GlobalP global,
				      z_NodeP node,
				      uint32 k);
/* Allocate the adjacent node with number k or return NULL.
   The adjacency oracle only needs to generate the children of a node.
   Generation of other adjacent nodes is allowed, but not necessary.
   Pre: 0 <= k < rs_Delta(global, node) */

typedef uint32 (*z_rs_FindChildNumberProc)(z_lb_GlobalP global,
					   z_NodeP child, z_NodeP parent);
/* Inverse function to the adjacency oracle: Return the first (and unique)
   integer k such that Adj(parent, k) == child. */

typedef void (*z_rs_ForwardProc)(z_lb_GlobalP global,
				 z_NodeP *nodeH,
				 uint32 *k);
/* This corresponds to a sequence of F, FindChildNumber, FreeNode */

typedef z_NodeP (*z_rs_NextChildProc)(z_lb_GlobalP global,
				      z_NodeP parent,
				      int k, int l, uint32 *num);

typedef struct z_rs_ProcSet {
  z_EqualProc			rs_Equal;
  z_rs_FProc			rs_F;
  z_rs_AdjOracleProc		rs_AdjOracle;
  z_ConditionProc		rs_IsRoot;
  z_rs_DeltaProc		rs_Delta;
  z_ConditionProc		rs_Condition; 	/* may be NULL */
  z_NodeProc			rs_Result; 	/* may be NULL */
  z_rs_FindChildNumberProc	rs_FindChildNumber; /* may be NULL */
  z_rs_ForwardProc		rs_Forward; 	/* may be NULL */
  z_rs_NextChildProc		rs_NextChild; 	/* may be NULL */
} z_rs_ProcSet, *z_rs_ProcSetP;

extern uint32 z_rs_cacheSize;

/* Procedures ----------------------------------------------------------- */

void z_rs_Enumerate(z_ProcSetNum procsetnum,
		    z_lb_GlobalP global,
		    z_NodeP root,
		    boolean sequential);
/* Enumerate all nodes starting from root (incl). Call
   rs_Result(node) if rs_Condition(node).
   */

void z_rs_Restart(z_ProcSetNum procsetnum,
		  z_lb_GlobalP global,
		  boolean sequential);


/* Initialization ------------------------------------------------------- */

void z_rs_Init(void);
/* Initialize this module */

#endif /* _Z_RS_H_ */
